/*@ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 **
 **    Copyright (c) 2007 - 2008, 
 **        Dave A. May [dave.mayhem23@gmail.com]
 **        School of Mathematical Sciences, 
 **        Monash University
 **        Victoria, 3800
 **        Australia
 **
 **    Copyright (c) 2009 - 2010, 
 **        Dave A. May [dave.mayhem23@gmail.com]
 **        Geophysical Fluid Dynamics, 
 **        Department of Earth Sciences,
 **        ETH Zürich,
 **        Switzerland
 **
 **    Project:       PetscExt_v3.1
 **    Description:   A collection of extensions to the PETSc library.
 **
 **    All rights reserved.
 **    Redistribution and use in source and binary forms, with or without modification,
 **    are permitted provided that the following conditions are met:
 **
 **        * Redistributions of source code must retain the above copyright notice,
 **              this list of conditions and the following disclaimer.
 **        * Redistributions in binary form must reproduce the above copyright
 **              notice, this list of conditions and the following disclaimer in the
 **              documentation and/or other materials provided with the distribution.
 **        * Neither the name of the Monash University nor the names of its contributors
 **              may be used to endorse or promote products derived from this software
 **              without specific prior written permission.
 **
 **    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 **    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 **    THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 **    PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 **    BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 **    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 **    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 **    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 **    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 **    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **
 **
 **    $Id$
 **
 ** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@*/


#include <petsc.h>
#include <petscvec.h>
#include <petscmat.h>
#include <private/matimpl.h>

#include "mat-symbolic_transpose-impl.h"
#include "private/mat/mat-symbolic_transpose.h"

#include "petscext_helpers.h"


/* 0*/
PetscErrorCode MatSetValues_SymTrans(Mat mat,PetscInt m,const PetscInt idxm[],PetscInt n,const PetscInt idxn[],const PetscScalar v[],InsertMode addv)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );

	PetscFunctionReturn(0);
}

PetscErrorCode MatGetRow_SymTrans(Mat mat,PetscInt row,PetscInt *ncols,PetscInt *cols[],PetscScalar *vals[])
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

PetscErrorCode MatRestoreRow_SymTrans(Mat mat,PetscInt row,PetscInt *ncols,PetscInt *cols[],PetscScalar *vals[])
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );

	PetscFunctionReturn(0);
}

PetscErrorCode MatMult_SymTrans(Mat mat,Vec x,Vec y)
{
	Mat_SymTrans   s = (Mat_SymTrans)mat->data;
	
	PetscFunctionBegin; 
	
	MatMultTranspose( s->A, x, y );
	
	PetscFunctionReturn(0);
}

PetscErrorCode MatMultAdd_SymTrans(Mat mat,Vec v1,Vec v2,Vec v3)
{
	Mat_SymTrans   s = (Mat_SymTrans)mat->data;
	
	PetscFunctionBegin; 
	
	MatMultTransposeAdd( s->A, v1, v2, v3 );
	
	PetscFunctionReturn(0);
}


/* 5*/
PetscErrorCode MatMultTranspose_SymTrans(Mat mat,Vec x,Vec y)
{
	Mat_SymTrans   s = (Mat_SymTrans)mat->data;
	
	PetscFunctionBegin;
	
	MatMult( s->A, x, y );
	
	PetscFunctionReturn(0);
}

PetscErrorCode MatMultTransposeAdd_SymTrans(Mat mat,Vec v1,Vec v2,Vec v3)
{
	Mat_SymTrans   s = (Mat_SymTrans)mat->data;
	
	PetscFunctionBegin; 
	
	MatMultAdd( s->A, v1, v2, v3 );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatSolve_SymTrans(Mat mat,Vec b,Vec x)
{
	Mat_SymTrans   s = (Mat_SymTrans)mat->data;
	
	PetscFunctionBegin; 
	
	MatSolveTranspose( s->A, b, x );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatSolveAdd_SymTrans(Mat mat,Vec b,Vec y,Vec x)
{
	Mat_SymTrans   s = (Mat_SymTrans)mat->data;
	
	PetscFunctionBegin; 
	
	MatSolveTransposeAdd( s->A, b, y, x );
	
	PetscFunctionReturn(0);
}

PetscErrorCode MatSolveTranspose_SymTrans(Mat mat,Vec b,Vec x)
{
	Mat_SymTrans   s = (Mat_SymTrans)mat->data;
	
	PetscFunctionBegin; 
	
	MatSolve( s->A, b, x );
	
	PetscFunctionReturn(0);
}

/*10*/
PetscErrorCode MatSolveTransposeAdd_SymTrans(Mat mat,Vec b,Vec y,Vec x)
{
	Mat_SymTrans   s = (Mat_SymTrans)mat->data;
	
	PetscFunctionBegin; 
	
	MatSolveAdd( s->A, b, y, x );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatLUFactor_SymTrans(Mat mat,IS row,IS col,MatFactorInfo *info)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatCholeskyFactor_SymTrans(Mat mat,IS perm,MatFactorInfo *info)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatRelax_SymTrans(Mat mat,Vec b,PetscReal omega,MatSORType flag,PetscReal shift,PetscInt its,PetscInt lits,Vec x)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatTranspose_SymTrans(Mat mat,MatReuse reuse,Mat *B)
{
	Mat_SymTrans   s = (Mat_SymTrans)mat->data;
	
	PetscFunctionBegin; 
	
	*B = s->A;
	PetscObjectReference( (PetscObject)s->A );
	
	PetscFunctionReturn(0);
}

/*15*/
PetscErrorCode MatGetInfo_SymTrans(Mat mat,MatInfoType flag,MatInfo *info)
{
	PetscErrorCode ierr;
	Mat_SymTrans   s = (Mat_SymTrans)mat->data;
	
	PetscFunctionBegin; 

	/* pull info out of the original operator */	
	ierr = MatGetInfo(s->A,flag,info);CHKERRQ(ierr);
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatEqual_SymTrans(Mat A,Mat B,PetscTruth *flg)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatGetDiagonal_SymTrans(Mat mat,Vec v)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatDiagonalScale_SymTrans(Mat mat,Vec l,Vec r)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatNorm_SymTrans(Mat mat,NormType type,PetscReal *nrm)
{
	Mat_SymTrans   s = (Mat_SymTrans)mat->data;
	PetscErrorCode ierr;
	
	PetscFunctionBegin; 
	
	ierr = MatNorm(s->A,type,nrm);CHKERRQ(ierr);
	
	PetscFunctionReturn(0);
}

/*20*/
PetscErrorCode MatAssemblyBegin_SymTrans(Mat mat,MatAssemblyType type)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatAssemblyEnd_SymTrans(Mat mat,MatAssemblyType type)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatCompress_SymTrans(Mat mat)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatSetOption_SymTrans(Mat mat,MatOption op)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatZeroEntries_SymTrans(Mat mat)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

/*25*/
PetscErrorCode MatZeroRows_SymTrans(Mat mat,PetscInt numRows,const PetscInt rows[],PetscScalar diag)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatLUFactorSymbolic_SymTrans(Mat mat,IS row,IS col,MatFactorInfo *info,Mat *fact)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatLUFactorNumeric_SymTrans(Mat mat,MatFactorInfo *info,Mat *fact)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatCholeskyFactorSymbolic_SymTrans(Mat mat,IS perm,MatFactorInfo *info,Mat *fact)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatCholeskyFactorNumeric_SymTrans(Mat mat,MatFactorInfo *info,Mat *fact)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

/*30*/
PetscErrorCode MatSetUpPreallocation_SymTrans(Mat B)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatILUFactorSymbolic_SymTrans(Mat mat,IS row,IS col,MatFactorInfo *info,Mat *fact)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatICCFactorSymbolic_SymTrans(Mat mat,IS perm,MatFactorInfo *info,Mat *fact)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatGetArray_SymTrans(Mat mat,PetscScalar *v[])
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatRestoreArray_SymTrans(Mat mat,PetscScalar *v[])
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

/*35*/
PetscErrorCode MatDuplicate_SymTrans(Mat mat,MatDuplicateOption op,Mat *M)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatForwardSolve_SymTrans(Mat mat,Vec b,Vec x)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatBackwardSolve_SymTrans(Mat mat,Vec b,Vec x)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatILUFactor_SymTrans(Mat mat,IS row,IS col,MatFactorInfo *info)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatICCFactor_SymTrans(Mat mat,IS row,MatFactorInfo* info)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

/*40*/
PetscErrorCode MatAXPY_SymTrans(Mat Y,PetscScalar a,Mat X,MatStructure str)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatGetSubMatrices_SymTrans(Mat mat,PetscInt n,const IS irow[],const IS icol[],MatReuse scall,Mat *submat[])
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatIncreaseOverlap_SymTrans(Mat mat,PetscInt n,IS is[],PetscInt ov)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatGetValues_SymTrans(Mat mat,PetscInt m,const PetscInt idxm[],PetscInt n,const PetscInt idxn[],PetscScalar v[])
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatCopy_SymTrans(Mat A,Mat B,MatStructure str)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

/*45*/
PetscErrorCode MatGetRowMax_SymTrans(Mat mat,Vec v,PetscInt idx[])
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatScale_SymTrans(Mat mat,PetscScalar a)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatShift_SymTrans( Mat mat, PetscScalar a )
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

PetscErrorCode MatDiagonalSet_SymTrans( Mat Y, Vec D, InsertMode is )
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatILUDTFactor_SymTrans(Mat mat,IS row,IS col,MatFactorInfo *info,Mat *fact)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

/*50*/
PetscErrorCode MatSetBlockSize_SymTrans(Mat mat,PetscInt bs)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatGetRowIJ_SymTrans(Mat mat,PetscInt shift,PetscTruth symmetric,PetscTruth blockcompressed,PetscInt *n,PetscInt *ia[],PetscInt* ja[],PetscTruth *done)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatRestorRowIJ_SymTrans(Mat mat,PetscInt shift,PetscTruth symmetric,PetscTruth blockcompressed,PetscInt *n,PetscInt *ia[],PetscInt* ja[],PetscTruth *done)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatGetColumnIJ_SymTrans(Mat mat,PetscInt shift,PetscTruth symmetric,PetscTruth blockcompressed,PetscInt *n,PetscInt *ia[],PetscInt* ja[],PetscTruth *done)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatRestorColumnIJ_SymTrans(Mat mat,PetscInt shift,PetscTruth symmetric,PetscTruth blockcompressed,PetscInt *n,PetscInt *ia[],PetscInt* ja[],PetscTruth *done)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

/*55*/

PetscErrorCode MatFDColoringCreate_SymTrans(Mat mat,ISColoring iscoloring,MatFDColoring color)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

PetscErrorCode MatColoringPatch_SymTrans(Mat mat,PetscInt ncolors,PetscInt n,ISColoringValue colorarray[],ISColoring *iscoloring)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatSetUnfactored_SymTrans(Mat mat)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}


PetscErrorCode MatPermute_SymTrans( Mat mat, IS row, IS col, Mat *B )
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatSetValuesBlocked_SymTrans(Mat mat,PetscInt m,const PetscInt idxm[],PetscInt n,const PetscInt idxn[],const PetscScalar v[],InsertMode addv)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

/*60*/
PetscErrorCode MatGetSubMatrix_SymTrans(Mat mat,IS isrow,IS iscol,PetscInt csize,MatReuse cll,Mat *newmat)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
/*
PetscErrorCode MatDestroy_SymTrans( Mat mat )
{
	PetscErrorCode ierr;
	
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
*/
PetscErrorCode MatView_SymTrans( Mat mat, PetscViewer viewer )
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatConvertFrom_SymTrans(Mat mat, MatType newtype,MatReuse reuse,Mat *M)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatUseScaledForm_SymTrans( Mat mat, PetscTruth scaled )
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

/*65*/
PetscErrorCode MatScaleSystem_SymTrans( Mat mat, Vec b, Vec x )
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatUnScaleSystem_SymTrans(Mat mat,Vec b,Vec x)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatSetLocalToGlobalMapping_SymTrans(Mat x,ISLocalToGlobalMapping mapping)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatSetValuesLocal_SymTrans(Mat mat,PetscInt nrow,const PetscInt irow[],PetscInt ncol,const PetscInt icol[],const PetscScalar y[],InsertMode addv)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatZeroRowsLocal_SymTrans(Mat mat,PetscInt numRows,const PetscInt rows[],PetscScalar diag)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

/*70*/
PetscErrorCode MatGetRowMaxAbs_SymTrans(Mat mat,Vec v,PetscInt idx[])
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatConvert_SymTrans(Mat mat, MatType newtype,MatReuse reuse,Mat *M)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatSetColoring_SymTrans( Mat mat, ISColoring colour )
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatSetValuesAdic_SymTrans(Mat mat,void *v)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatSetValuesAdifor_SymTrans(Mat mat,PetscInt nl,void *v)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}


/*75*/
PetscErrorCode MatFDColoringApply_SymTrans(Mat J,MatFDColoring coloring,Vec x1,MatStructure *flag,void *sctx)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatSetFromOptions_SymTrans( Mat mat )
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatMultConstrained_SymTrans(Mat mat,Vec x,Vec y)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatMultTransposeConstrained_SymTrans(Mat mat,Vec x,Vec y)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatILUFactorSymbolicConstrained_SymTrans(Mat mat,IS row,IS col,double d,PetscInt i,PetscInt j,Mat *fact)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}


/*80*/
PetscErrorCode MatPermuteSparsify_SymTrans(Mat A, PetscInt band, PetscReal frac, PetscReal tol, IS rowp, IS colp, Mat *B)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatMults_SymTrans(Mat A,Vecs x,Vecs y)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatSolves_SymTrans(Mat A,Vecs b,Vecs x)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatGetInertia_SymTrans(Mat mat,PetscInt *nneg,PetscInt *nzero,PetscInt *npos)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatLoad_SymTrans(PetscViewer viewer, MatType outtype,Mat *newmat)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}


/*85*/
PetscErrorCode MatIsSymmetric_SymTrans(Mat A,PetscReal tol,PetscTruth *flg)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatIsHermitian_SymTrans(Mat A,PetscTruth *flg)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatIsStructurallySymmetric_SymTrans(Mat A,PetscTruth *flg)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatPBRelax_SymTrans(Mat mat,Vec b,PetscReal omega,MatSORType flag,PetscReal shift,PetscInt its,PetscInt lits,Vec x)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatGetVecs_SymTrans(Mat mat,Vec *right,Vec *left)
{
	Mat_SymTrans   s = (Mat_SymTrans)mat->data;
	
	PetscFunctionBegin; 
	
	MatGetVecs( s->A, left, right );
	
	PetscFunctionReturn(0);
}



/*90*/
PetscErrorCode MatMatMult_SymTrans(Mat A,Mat B,MatReuse scall,PetscReal fill,Mat *C) 
{
	Mat_SymTrans   sA;
	Mat _A, _B;
	PetscTruth is_Asym;
	MPI_Comm comm;
	PetscInt M,N;	

	PetscFunctionBegin; 
	
	/* Special cases, 
		i)  A=symBt => use Pt I P, where P = B
	*/
	
	is_Asym = PETSC_FALSE;
	PetscTypeCompare( (PetscObject)A, "symtrans", &is_Asym );
	if( is_Asym == PETSC_TRUE ) {
		sA = (Mat_SymTrans)A->data;
		
		if( sA->A == B ) {
			/* Build identity */
			Mat Identity;
			
			PetscObjectGetComm( (PetscObject)B, &comm );
			MatGetSize( B, &M, &N );
			Identity = MATAIJIdentityNew( comm, M ); 
			MatPtAP( Identity, B, scall, fill, C );
			MatDestroy( Identity );
			
			PetscFunctionReturn(0);
		}
	}
	
	MatSymTransGetExplicitOperator( A, &_A );
	MatSymTransGetExplicitOperator( B, &_B );
	
	MatMatMult( _A, _B, scall, fill, C );
	
	MatDestroy( _B );
	MatDestroy( _A );
	
	
	
	PetscFunctionReturn(0);
	
}
PetscErrorCode MatMatMultSymbolic_SymTrans(Mat A,Mat B,PetscReal fill,Mat *C) 
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatMatMultNumeric_SymTrans(Mat A,Mat B,Mat C)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatPtAP_SymTrans(Mat A,Mat P,MatReuse scall,PetscReal fill,Mat *C) 
{
	Mat _A, _P;
	
	PetscFunctionBegin; 
	
	
	MatSymTransGetExplicitOperator( A, &_A );
	MatSymTransGetExplicitOperator( P, &_P );
	
	MatPtAP( _A, _P, scall, fill, C );
	
	MatDestroy( _P );
	MatDestroy( _A );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatPtAPSymbolic_SymTrans(Mat A,Mat P,PetscReal fill,Mat *C) 
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}


/*95*/
PetscErrorCode MatPtAPNumeric_SymTrans(Mat A,Mat P,Mat C) 
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatMatMultTranspose_SymTrans(Mat A,Mat B,MatReuse scall,PetscReal fill,Mat *C) 
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatMatMultTransposeSymbolic_SymTrans(Mat A,Mat B,PetscReal fill,Mat *C) 
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatMatMultTransposeNumeric_SymTrans(Mat A,Mat B,Mat C)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatPtAPSymbolic__SEQAIJ_SymTrans(Mat A,Mat P,PetscReal fill,Mat *C) 
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}


/*100*/
PetscErrorCode MatPtAPNumeric_SEQAIJ_SymTrans(Mat A,Mat P,Mat C) 
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatPtAPSymbolic_MPIAIJ_SymTrans(Mat A,Mat P,PetscReal fill,Mat *C) 
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatPtAPNumeric_MPIAIJ_SymTrans(Mat A,Mat P,Mat C) 
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatConjugate_SymTrans(Mat mat)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatSetSizes_SymTrans(Mat mat,PetscInt m,PetscInt n,PetscInt M,PetscInt N)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}


/*105*/
PetscErrorCode MatSetValuesRow_SymTrans(Mat mat,PetscInt row,const PetscScalar v[])
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatRealPart_SymTrans(Mat mat)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatImaginaryPart_SymTrans(Mat mat)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

PetscErrorCode MatGetRowUpperTriangular_SymTrans(Mat mat)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

PetscErrorCode MatRestroreRowUpperTriangular_SymTrans(Mat mat)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}


/*110*/
PetscErrorCode MatMatSolve_SymTrans(Mat mat,Mat b,Mat x)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatGetRedundantMatrix_SymTrans(Mat mat,PetscInt nsubcomm,MPI_Comm subcomm,PetscInt mlocal_red,MatReuse reuse,Mat *matredundant)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatGetRowMin_SymTrans(Mat mat,Vec v,PetscInt idx[])
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}
PetscErrorCode MatGetColumnVector_SymTrans(Mat A,Vec yy,PetscInt col)
{
	PetscFunctionBegin; SETERRQ1( PETSC_ERR_SUP,"Operation: %s is not implemented", __func__ );
	
	PetscFunctionReturn(0);
}

